/*
 * Copyright (c) 2008 - 2012, Andy Bierman, All Rights Reserved.
 * Copyright (c) 2012 - 2020, YumaWorks, Inc., All Rights Reserved.
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *

*** Generated by yangdump-sdk

    Combined SIL module
    module notif-example
    revision 2019-01-01
    namespace http://netconfcentral.org/ns/notif-example
    CLI parameters:
        format c
        indent 4
        log-level debug3
        module notif-example
        output notif-example.c
        sil-edit2
        sil-get2
        unified true

 */

#include <xmlstring.h>

#include "procdefs.h"
#include "agt.h"
#include "agt_action.h"
#include "agt_cb.h"
#include "agt_cfg.h"
#include "agt_not.h"
#include "agt_rpc.h"
#include "agt_sil_lib.h"
#include "agt_timer.h"
#include "agt_util.h"
#include "agt_val.h"
#include "cfg.h"
#include "dlq.h"
#include "getcb.h"
#include "ncx.h"
#include "ncx_feature.h"
#include "ncxmod.h"
#include "ncxtypes.h"
#include "obj.h"
#include "rpc.h"
#include "ses.h"
#include "status.h"
#include "val.h"
#include "val_child.h"
#include "val_util.h"
#include "xml_util.h"
#include "xml_val.h"
#include "notif-example.h"

/* module static variables */
static ncx_module_t *notif_example_mod;
static obj_template_t *restart_notification_obj;

/* put your static variables here */

/********************************************************************
* FUNCTION y_notif_example_init_static_vars
*
* initialize module static variables
*
********************************************************************/
static void y_notif_example_init_static_vars (void)
{
    notif_example_mod = NULL;
    restart_notification_obj = NULL;

    /* init your static variables here */

} /* y_notif_example_init_static_vars */


/********************************************************************
* FUNCTION test_notif_validate
*
* RPC validation phase
* All YANG constriants have passed at this point.
* Add description-stmt checks in this function.
*
* INPUTS:
*     see agt/agt_rpc.h for details
*
* RETURNS:
*     error status
********************************************************************/
static status_t
    test_notif_validate (ses_cb_t *scb,
                         rpc_msg_t *msg,
                         xml_node_t *methnode)
{
    status_t res = NO_ERR;
    val_value_t *errorval = NULL;;
    const xmlChar *errorstr = NULL;;

    /* if error: set the res, errorstr, and errorval parms */
    if (res != NO_ERR) {
        agt_record_error(scb,
                         &msg->mhdr,
                         NCX_LAYER_OPERATION,
                         res,
                         methnode,
                         NCX_NT_STRING,
                         errorstr,
                         NCX_NT_VAL,
                         errorval);
    }

    return res;

} /* test_notif_validate */


/********************************************************************
* FUNCTION y_toaster_cancel_toast_invoke
*
* RPC invocation phase
* All constraints have passed at this point.
* Call device instrumentation code in this function.
*
* INPUTS:
*     see agt/agt_rpc.h for details
*
* RETURNS:
*     error status
********************************************************************/
static status_t
    test_notif_invoke (ses_cb_t *scb,
                       rpc_msg_t *msg,
                       xml_node_t *methnode)
{
    status_t res;

    res = NO_ERR;

    /* remove the next line if scb is used */
    (void)scb;

    /* remove the next line if methnode is used */
    (void)methnode;

    /* invoke your device instrumentation code here */

    /* Send the notification */
    restart_notification_send(msg);

    return res;

} /* y_toaster_cancel_toast_invoke */


/********************************************************************
* FUNCTION restart_notification_send
*
* Send a y_notif_example_restart_notification notification
* Called by your code when notification event occurs
*
********************************************************************/
void
    restart_notification_send (rpc_msg_t *msg)
{
    status_t res = NO_ERR;

    if (!agt_notifications_enabled()) {
        log_debug2("\nSkipping <restart-notification> notification; disabled");
        return;
    }

    if (LOGDEBUG) {
        log_debug("\nGenerating <restart-notification> notification");
    }

    /* Create a new notification message */
    agt_not_msg_t *notif =
                                 // loaded obj in the Init 1 Phase
        agt_not_new_notification(restart_notification_obj);
    if (notif == NULL) {
        log_error("\nError: malloc failed, cannot send "
                  "<restart-notification> notification");
        return;
    }

    /* add container resource to payload */
    val_value_t *resource_val =
        agt_make_object(restart_notification_obj,
                        (const xmlChar *)"resource",
                        &res);
    if (!resource_val) {
        log_error("\nError: make container failed (%s), cannot send "
                  "<resourse> parameter",
                  get_error_string(res));
        res = NO_ERR;
    } else {
        /* pass off resource_val malloc here */
        agt_not_add_to_payload(notif, resource_val);
    }

    /* create a child of a container and then add it to container */
    if (resource_val) {
        val_value_t *leafval =
            agt_make_leaf(VAL_OBJ(resource_val),
                          (const xmlChar *)"connection-name",
                          (const xmlChar *)"connection-1",
                          &res);
        if (leafval) {
            res = val_child_add(leafval, resource_val);
            if (res != NO_ERR) {
                val_free_value(leafval);
            }
        }
    }

    /* add 'target' list nodes
     * In this example we use custom audit records that
     * are createad by the server. The record structure is just an
     * example and is out of the scope of this example code.
     * Use your own data or structure or other source of the data
     * to generate list entries.
     *
     * Here we loop through the data massive and generate the
     * notification list values.
     */
    if (msg->rpc_txcb) {

        agt_cfg_audit_rec_t *auditrec =
            (agt_cfg_audit_rec_t *)dlq_firstEntry(&msg->rpc_txcb->auditQ);
        for ( ; auditrec;
                auditrec = (agt_cfg_audit_rec_t *)dlq_nextEntry(auditrec)) {

            /* add restart-notification/target list */
            val_value_t *listval =
                agt_make_object(restart_notification_obj,
                                (const xmlChar *)"target",
                                &res);
            if (listval == NULL) {
                log_error("\nError: make list failed (%s), cannot send "
                          "<target> parameter",
                          get_error_string(res));
                continue;
            }

            /* pass off listval malloc here */
            agt_not_add_to_payload(notif, listval);

            /* add restart-notification/target/target-name */
            val_value_t *leafval =
                agt_make_leaf(VAL_OBJ(listval),
                              (const xmlChar *)"target-name",
                              (const xmlChar *)"somename",
                              &res);
            if (leafval) {
                res = val_child_add(leafval, listval);
                if (res != NO_ERR) {
                    val_free_value(leafval);
                }
            } else {
                log_error("\nError: make leaf failed (%s), cannot send "
                          "<target-name> parameter",
                          get_error_string(res));
            }
        }
    }

    agt_not_queue_notification(notif);

} /* restart_notification_send */


/********************************************************************
* FUNCTION y_notif_example_init
*
* initialize the notif-example server instrumentation library
*
* INPUTS:
*    modname == requested module name
*    revision == requested version (NULL for any)
*
* RETURNS:
*     error status
********************************************************************/
status_t
    y_notif_example_init (const xmlChar *modname,
                          const xmlChar *revision)
{
    status_t res = NO_ERR;

    y_notif_example_init_static_vars();

    /* change if custom handling done */
    if (xml_strcmp(modname, y_notif_example_M_notif_example)) {
        return ERR_NCX_UNKNOWN_MODULE;
    }

    if (revision && xml_strcmp(revision, y_notif_example_R_notif_example)) {
        return ERR_NCX_WRONG_VERSION;
    }

    res =
        ncxmod_load_module(y_notif_example_M_notif_example,
                           y_notif_example_R_notif_example,
                           agt_get_savedevQ(),
                           &notif_example_mod);
    if (res != NO_ERR) {
        return res;
    }

    restart_notification_obj =
        ncx_find_object(notif_example_mod,
                        y_notif_example_N_restart_notification);
    if (restart_notification_obj == NULL) {
        return ERR_NCX_DEF_NOT_FOUND;
    }

    res =
        agt_rpc_register_method(y_notif_example_M_notif_example,
                                (const xmlChar *)"test-notif",
                                AGT_RPC_PH_VALIDATE,
                                test_notif_validate);
    if (res != NO_ERR) {
        return res;
    }

    res =
        agt_rpc_register_method(y_notif_example_M_notif_example,
                                (const xmlChar *)"test-notif",
                                AGT_RPC_PH_INVOKE,
                                test_notif_invoke);
    if (res != NO_ERR) {
        return res;
    }

    /* put your module initialization code here */

    return res;

} /* y_notif_example_init */


/********************************************************************
* FUNCTION y_notif_example_init2
*
* SIL init phase 2: non-config data structures
* Called after running config is loaded
*
* RETURNS:
*     error status
********************************************************************/
status_t y_notif_example_init2 (void)
{
    status_t res = NO_ERR;

    /* put your init2 code here */

    return res;

} /* y_notif_example_init2 */

/********************************************************************
* FUNCTION y_notif_example_cleanup
*    cleanup the server instrumentation library
*
********************************************************************/
void y_notif_example_cleanup (void)
{
    agt_rpc_unregister_method(y_notif_example_M_notif_example,
                              (const xmlChar *)"test-notif");

    /* put your cleanup code here */

} /* y_notif_example_cleanup */

/* END SIL notif_example.c */
